﻿class CapabilityLocations {
  lang = '';
  data = [];
  capability = 0;
  el = null;
  id = 0;
  groups = [
    //1
    {
      en: 'Cosma Locations',
      de: 'Cosma-Standorte',
      es: 'Ubicaciones de Cosma',
      zh: '卡斯马办公地点',
      ja: 'コズマの所在地',
    },
    //2
    {
      en: 'Magna Powertrain Locations',
      de: 'Magna Powertrain-Standorte',
      es: 'Ubicaciones de Magna Powertrain',
      zh: '麦格纳动力总成办公地点',
      ja: 'マグナ・パワートレーンの所在地',
    },
    //3
    {
      en: 'Magna Exteriors Locations',
      de: 'Magna Exteriors-Standorte',
      es: 'Ubicaciones de Magna Exteriors',
      zh: '麦格纳外饰系统办公地点',
      ja: 'マグナ・エクステリアーズの所在地',
    },
    //4
    {
      en: 'Magna Seating Locations',
      de: 'Magna Seating-Standorte',
      es: 'Ubicaciones de Magna Seating',
      zh: '麦格纳座椅系统办公地点',
      ja: 'マグナ・シーティングの所在地',
    },
    //5
    {
      en: '',
      de: '',
      es: '',
      zh: '',
      ja: '',
    },
    //6
    {
      en: 'Magna Mirrors Locations',
      de: 'Magna Mirrors-Standorte',
      es: 'Ubicaciones de Magna Mirrors',
      zh: '麦格纳后视镜系统办公地点',
      ja: 'マグナ・ミラーズの所在地',
    },
    //7
    {
      en: 'Mechatronics Locations',
      de: 'Mechatronik-Standorte',
      es: 'Ubicaciones de Mecatrónica',
      zh: '机电智能系统办公地点',
      ja: 'メカトロニクスの所在地',
    },
    //8
    {
      en: '',
      de: '',
      es: '',
      zh: '',
      ja: '',
    },

    //9
    {
      en: 'Magna Electronics Locations',
      de: 'Magna Electronics-Standorte',
      es: 'Ubicaciones de Magna Electronics',
      zh: '麦格纳电子系统办公地点',
      ja: 'マグナ・エレクトロニクスの所在地',
    },
    //10
    {
      en: 'Magna Steyr Locations',
      de: 'Magna Steyr-Standorte',
      es: 'Ubicaciones de Magna Steyr',
      zh: '麦格纳斯太尔办公地点',
      ja: 'マグナ・シュタイヤーの所在地',
    },
    //11
    {
      en: '',
      de: '',
      es: '',
      zh: '',
      ja: '',
    },
    //12
    {
      en: 'Magna Lighting Locations',
      de: 'Magna Lighting-Standorte',
      es: 'Ubicaciones de Magna Lighting',
      zh: '麦格纳照明系统办公地点',
      ja: 'マグナ・ライティングの所在地',
    },
  ];

  /**
   * @param {Element} el - The root element of the widget.
   * @param {number} capability - The capability value.
   */
  constructor(el, capability) {
    /**
     * The language code.
     * @type {string}
     */
    this.lang = document.documentElement.lang;

    /**
     * The data for locations.
     * @type {Array}
     */
    this.data = [];

    /**
     * The capability value.
     * @type {number}
     */
    this.capability = capability;

    /**
     * The root element of the widget.
     * @type {Element}
     */
    this.el = el;

    /**
     * The unique ID of the widget.
     * @type {number}
     */
    this.id = Math.ceil(Math.random() * Date.now());
  }

  /**
   * Finds an object in the data by matching the key and ID.
   * @param {Object} o - The object to search within.
   * @param {string} key - The key to match.
   * @param {number} id - The ID to match.
   * @returns {Object} - The matched object, or undefined if not found.
   */
  findById(o, key, id) {
    // Early return if the current object matches the ID
    if (o && o[key] === id) {
      return o;
    }

    let result, p;
    for (p in o) {
      if (o.hasOwnProperty(p) && typeof o[p] === 'object') {
        result = this.findById(o[p], key, id);
        if (result) {
          return result;
        }
      }
    }

    return result;
  }

  loadData() {
    const $ = window.jQuery;
    const _this = this;
    const apiUrl = `/magnaapi/FacilitiesMapData/getfacilities/?capability=${_this.capability}&lang=${_this.lang}`;

    $.ajax(apiUrl)
      .done(function (response) {
        _this.data = $.parseJSON(response);
        _this.initApp();
      })
      .fail(function () {
        console.error('Uable to load Locations data');
      });
  }

  initApp() {
    const $ = window.jQuery;
    const _this = this;

    let tpl = ``;
    let str = `
        <div class="c-locations">
            <div class="c-locations__stage">
                <ul class="c-locations__panel o-list" aria-hidden="false">`;

    let country_str = '';

    $.each(this.data, function (i, continent) {
      // Enumerate through available countries
      if (continent.country.length) {
        str += `
            <li class="c-locations__item" aria-expanded="false">
                <button class="c-locations__link c-locations__link--level1 o-buttonlink" type="button" aria-controls="continent_${i}_${_this.id}">
                    ${continent.regionName}
                </button>
                <nav class="c-locations__wrapper" id="continent_${i}_${_this.id}">
                    <ul class="c-locations__group o-list">`;

        // Enumerate through available locations
        $.each(continent.country, function (j, country) {
          if (country.location.length) {
            str += `<li class="c-locations__item" aria-expanded="false">
                <button class="c-locations__link c-locations__link--level2 o-button o-button--link" type="button" aria-controls="group_${country.country_id}_${_this.id}">
                    ${country.countryName}
                </button>
            </li>`;

            country_str += `<nav class="c-locations__panel" id="group_${country.country_id}_${_this.id}" aria-hidden="true">
                <ul class="c-locations__group o-list">
                    <li class="c-locations__item" aria-expanded="false">
                        <span class="c-locations__link c-locations__link--level1 o-buttonlink">
                            ${country.countryName}
                        </span>
                    </li>`;

            $.each(country.location, function (k, location) {
              country_str += `<li class="c-locations__item">`;

              if (!location.FacilityName && !location.address) {
                country_str += `
                    <span class="c-locations__link o-button o-button--link" style="cursor: auto; display: block;">
                        ${location.LocationName}
                        <span class="c-locations__legend">`;

                $.each(location.facility, function (l, facility) {
                  if (l) {
                    country_str += ', ';
                  }
                  country_str += facility.typeName;
                });
                country_str += '</span></span>';
              } else {
                country_str += `
                    <button class="c-locations__link c-locations__link--level3 o-button o-button--link" type="button" aria-controls="${this.id}_details_${_this.id}" data-rel="${location.facilityId}" style="display: block;">
                        ${location.LocationName}
                        <span class="c-locations__legend">`;

                $.each(location.facility, function (l, facility) {
                  if (l) {
                    country_str += ', ';
                  }
                  country_str += facility.typeName;
                });
                country_str += '</span></button>';
              }
              country_str += '</li>';
            });

            country_str += '</ul></nav>';
          }
        });

        str += '</ul></nav></li>';
      }
    });

    str += `
        </ul>
        
        ${country_str}    
        
        <button class="c-locations__back o-button o-button--link" type="button" aria-hidden="true">
            <span class="o-a11y__visuallyhidden">Back</span>
            <svg class="spacing-top-0" xmlns="http://www.w3.org/2000/svg" viewBox="0 0 43.7 57.9"><path d="M21.2 57.9L0 29 21.2 0 24.4 2.4 5 29 24.4 55.6z"/><path d="M3.7 27H43.7V31H3.7z"/></svg>
        </button>
    </div>
    
    <div class="c-locations__details" id="c_locations_details_${_this.id}" aria-hidden="true"></div>
    </div>`;

    // Initialize widget
    tpl = str;
    _this.el.innerHTML = tpl;

    const is_animation_supported =
      typeof CSS !== 'undefined' && CSS.supports('animation-name', 'foo');
    const tree = _this.el.querySelector('.c-locations');
    const stage = _this.el.querySelector('.c-locations__stage');
    const back_button = _this.el.querySelector('.c-locations__back');
    const level1_nav = _this.el.querySelectorAll('.c-locations__link--level1');
    const level2_nav = _this.el.querySelectorAll('.c-locations__link--level2');
    const location_nav = _this.el.querySelectorAll(
      '.c-locations__link--level3'
    );
    const details = _this.el.querySelector(`#c_locations_details_${_this.id}`);
    const panels = _this.el.querySelectorAll('.c-locations__panel');

    function panelAnimationEnd(e) {
      // console.log('animationend', e);
      if (e.animationName.indexOf('fade-out-') > -1) {
        e.target.setAttribute('aria-hidden', true);
      }
    }

    function detailsAnimationEnd(e) {
      // Exiting canvas
      if (e.animationName.indexOf('fade-out-') > -1) {
        const details_width = details.offsetWidth / 2;

        e.target.setAttribute('aria-hidden', true);

        stage.style.left = -details_width + 'px';
        stage.style.transition =
          'transform 0.25s cubic-bezier(0.390, 0.575, 0.565, 1.000)';
        stage.style.transform = 'translateX(' + details_width + 'px)';
      }
    }

    function stageTransitionEnd(e) {
      if (e.propertyName === 'transform') {
        e.target.style.transition = '';
        e.target.style.transform = '';
        e.target.style.left = 'auto';
        // console.info('stage reset');
      }
    }

    /**
     * Change stage based on ID.
     * @param {Element} el
     */
    function changeStage(el) {
      if (el && el.nodeType) {
        for (var x = 0; x < panels.length; x++) {
          if (
            panels[x] !== el &&
            panels[x].classList.contains('c-locations__panel')
          ) {
            if (is_animation_supported) {
              panels[x].style.animation =
                'fade-out-left 0.6s cubic-bezier(0.250, 0.460, 0.450, 0.940) both';
            } else {
              panels[x].setAttribute('aria-hidden', true);
            }
          }
        }

        el.setAttribute('aria-hidden', false);

        if (is_animation_supported) {
          el.style.animation =
            'fade-in-right 0.6s 0.2s cubic-bezier(0.390, 0.575, 0.565, 1.000) both';
        }
      }
    }

    /**
     * Explore available continents.
     *
     * @param {Event} e
     */
    function toggleContinentSubNav(e) {
      const nav = e.target.nextElementSibling; // .c-locations__wrapper
      const siblings = e.target.parentElement.parentElement.children;
      const nav_height = nav.firstElementChild.offsetHeight; // .c-locations__group

      nav.style.setProperty('--groupheight', nav_height + 'px');

      for (var x = 0; x < siblings.length; x++) {
        siblings[x].setAttribute('aria-expanded', false);
      }

      this.parentElement.setAttribute('aria-expanded', true);
    }

    /**
     * Explore available countries.
     *
     * @param {Event} e
     */
    function clickLevel2(e) {
      const nav_id = e.target.getAttribute('aria-controls');
      const nav_el = document.querySelector('#' + nav_id);
      changeStage(nav_el);
      back_button.setAttribute('aria-hidden', false);
    }

    /**
     * Retrieve location data.
     *
     * @param {Event} e
     */
    function loadLocation(e) {
      let str = '';
      const facilityId = this.getAttribute('data-rel');

      const location = _this.findById(_this.data, 'facilityId', facilityId);

      // If there is no facility data, do not bother showing the final "details" stage.
      if (!location.FacilityName && !location.address) {
        return;
      }

      const nav_siblings = this.parentElement.parentElement.childNodes;

      // Lit state
      this.classList.add('js-active');
      this.classList.remove('js-inactive');

      for (var x = 1; x < nav_siblings.length; x++) {
        if (this !== nav_siblings[x].firstElementChild) {
          nav_siblings[x].firstElementChild.classList.add('js-inactive');
          nav_siblings[x].firstElementChild.classList.remove('js-active');
        }
      }

      str += '<img alt="" src="' + location.photo + '">';
      str +=
        '<h3 class="u-fontweight-100 u-margin0--top">' +
        location.FacilityName +
        '</h3>';
      str +=
        '<address style="font-style: normal;">' +
        location.address +
        '</address>';

      details.innerHTML = str;

      if (details.getAttribute('aria-hidden') === 'true') {
        details.setAttribute('aria-hidden', false);

        const details_width = details.offsetWidth / 2;

        if (is_animation_supported) {
          details.style.animation =
            'fade-in-right 0.6s 0.2s cubic-bezier(0.390, 0.575, 0.565, 1.000) both';
          stage.style.left = details_width + 'px';
          stage.style.transition =
            'transform 0.6s cubic-bezier(0.390, 0.575, 0.565, 1.000)';
          stage.style.transform = 'translateX(-' + details_width + 'px)';
        }
      }
    }

    /**
     * Event listeners
     */
    for (var i = 0; i < level1_nav.length; i++) {
      level1_nav[i].addEventListener('click', toggleContinentSubNav);
    }

    for (var i = 0; i < level2_nav.length; i++) {
      level2_nav[i].addEventListener('click', clickLevel2);
    }

    for (var i = 0; i < location_nav.length; i++) {
      location_nav[i].addEventListener('click', loadLocation);
    }

    if (is_animation_supported) {
      for (var i = 0; i < panels.length; i++) {
        panels[i].addEventListener('animationend', panelAnimationEnd);
      }
      details.addEventListener('animationend', detailsAnimationEnd);
      stage.addEventListener('transitionend', stageTransitionEnd);
    }

    back_button.addEventListener('click', function () {
      changeStage(stage.firstElementChild);
      if (is_animation_supported) {
        details.style.animation =
          'fade-out-right 0.25s cubic-bezier(0.250, 0.460, 0.450, 0.940) both';
      }
      back_button.setAttribute('aria-hidden', true);
    });
  }
}
